@@ -612,28 +612,35 @@ class PostScriptDevice(CommonDevice):
 
     def TileImage(self, image, trafo):
 	width, height = image.size
-	length = width * height * len(image.mode)
+        if image.mode == 'RGBA':
+            # We don't support transparency in textures, so we simply
+            # treat RGBA as as RGB images
+            mode = 'RGB'
+        else:
+            mode = image.mode
+	length = width * height * len(mode)
 	if length > 65536:
 	    # the tile image is too large to fit into a PostScript
 	    # string. Resize it.
 	    #warn(USER, "Image data to big for tiling, resizing...")
 	    ratio = float(width) / height
-	    max_size = 65536 / len(image.mode)
+	    max_size = 65536 / len(mode)
 	    width = int(sqrt(max_size * ratio))
 	    height = int(sqrt(max_size / ratio))
 	    tile = image.im.resize((width, height))
-	    length = width * height * len(image.mode)
+	    length = width * height * len(mode)
 	    trafo = trafo(Scale(float(image.size[0]) / width,
 				float(image.size[1]) / height))
 	else:
 	    tile = image.im
 	write = self.file.write
 	write('%d %d %d [%g %g %g %g %g %g]\n'
-	      % ((width, height, len(image.mode)) + trafo.coeff()))
+	      % ((width, height, len(mode)) + trafo.coeff()))
 	digits = length * 2	# 2 digits per byte
 	lines = (digits - 1) / 80 + 1
 	write('%%%%BeginData: %d Hex Lines\n' % (lines + 1))
 	write("tileimage\n")
+        # write_ps_hex treats RGBA like RGB
 	_sketch.write_ps_hex(tile, self.file)
 	write('%%EndData\n')
 
