@@ -1,5 +1,5 @@
 # Sketch - A Python-based interactive drawing program
-# Copyright (C) 1997, 1998, 1999, 2000 by Bernhard Herzog
+# Copyright (C) 1997, 1998, 1999, 2000, 2001 by Bernhard Herzog
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,7 +22,7 @@ from Sketch.Lib import util
 from Sketch.warn import warn, warn_tb, INTERNAL, USER
 from Sketch import _, config, load, plugins, SketchVersion
 from Sketch import Publisher, Point, EmptyFillStyle, EmptyLineStyle, \
-     EmptyPattern, Document, GuideLine, PostScriptDevice, SketchLoadError
+     EmptyPattern, Document, GuideLine, PostScriptDevice, SketchError
 import Sketch
 from Sketch.Graphics import image, eps
 import Sketch.Scripting
@@ -197,7 +197,7 @@ class SketchMainWindow(Publisher):
 	    doc = load.load_drawing(filename)
 	    self.SetDocument(doc)
 	    self.add_mru_file(filename)
-	except SketchLoadError, value:
+	except SketchError, value:
 	    app.MessageBox(title = _("Open"),
 			   message = _("An error occurred:\n") + str(value))
 	    #warn_tb(USER, "An error occurred:\n", str(value))
@@ -397,7 +397,7 @@ class SketchMainWindow(Publisher):
 		filename = os.path.join(os.getcwd(), filename)
 	    doc = load.load_drawing(filename)
 	    group = doc.as_group()
-	except SketchLoadError, value:
+	except SketchError, value:
 	    app.MessageBox(title = _("Insert Document"),
 			   message = _("An error occurred:\n") + str(value))
 	    #warn_tb(USER, "An error occurred:\n", str(value))
